@@ -1,3 +1,4 @@
+from collections import OrderedDict
 from pathlib import Path
 import subprocess
 
@@ -34,7 +35,7 @@ def load_installed_challenges():
     return s.get("/api/v1/challenges?view=admin", json=True).json()["data"]
 
 
-def sync_challenge(challenge, ignore=[]):
+def sync_challenge(challenge, ignore=[], ctfd_name=None):
     data = {
         "name": challenge["name"],
         "category": challenge["category"],
@@ -59,7 +60,8 @@ def sync_challenge(challenge, ignore=[]):
 
     installed_challenges = load_installed_challenges()
     for c in installed_challenges:
-        if c["name"] == challenge["name"]:
+        challenge_name = ctfd_name if ctfd_name is not None else challenge["name"]
+        if c["name"] == challenge_name:
             if "category" in ignore:
                 data["category"] = c["category"]
             challenge_id = c["id"]
@@ -566,8 +568,7 @@ def pull_challenge(challenge, ignore=(), update_files=False, create_files=False,
         return
     
     remote_details = get_challenge_details(challenge_id)
-    # Ignore optional fields with default values
-    remote_details = {k: v for k, v in remote_details.items() if not is_default(k, v) or create_defaults}
+
     local_files = {Path(f).name : f for f in challenge.get("files",[])}
     # Update files
     for f in remote_details["files"]:
@@ -579,6 +580,8 @@ def pull_challenge(challenge, ignore=(), update_files=False, create_files=False,
             req = s.get(f)
             req.raise_for_status()
             Path(challenge.directory, f_base).write_bytes(req.content)
+            if "files" not in challenge:
+                challenge["files"] = []
             challenge["files"].append(f_base)
 
         elif f_base in local_files and update_files:
@@ -597,15 +600,34 @@ def pull_challenge(challenge, ignore=(), update_files=False, create_files=False,
         
     print(f"Updating challenge.yml for {challenge['name']}")
 
+    # Prefer ordering remote details as described by spec
+    preferred_order = ["name", "category", "description", \
+        "value", "type", "connection_info", "attempts", \
+            "flags", "topics", "tags", "files", "hints", "requirements", "state"]
+
     # Merge local and remote challenge.yml & Preserve local keys + order
     updated_challenge = dict(challenge)
 
-    for key in remote_details:
-        if key in ignore:
-            continue
-        updated_challenge[key] = remote_details[key]
+    # Ignore optional fields with default values
+    remote_details_updates = {}
+    for k, v in remote_details.items():
+        # If the key value changed, we want to update it
+        if k in challenge and challenge[k] != v:
+            remote_details_updates[k] = v
+        elif not is_default(k, v) or create_defaults:
+            remote_details_updates[k] = v
 
-    # Hack: remove whitespace before newlines in multiline strings
+    # Add all preferred keys
+    for key in preferred_order:
+        if key in remote_details_updates and key not in ignore:
+            updated_challenge[key] = remote_details_updates[key]
+
+    # Add remaining keys
+    for key in remote_details_updates:
+        if key not in preferred_order and key not in ignore:
+            updated_challenge[key] = remote_details_updates[key]
+
+    # Hack: remove tabs in multiline strings
     updated_challenge['description'] = updated_challenge['description'].replace('\t', '')
 
     with open(challenge.file_path, "w") as f:
